@@ -33,8 +33,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/mergepatch"
 	"k8s.io/apimachinery/pkg/util/strategicpatch"
-	gatewayv1 "sigs.k8s.io/gateway-api/apis/v1"
-	gateway "sigs.k8s.io/gateway-api/apis/v1beta1"
+	gateway "sigs.k8s.io/gateway-api/apis/v1"
 	gatewayx "sigs.k8s.io/gateway-api/apisx/v1alpha1"
 	"sigs.k8s.io/yaml"
 
@@ -157,6 +156,10 @@ func getBuiltinClasses() map[gateway.ObjectName]gateway.GatewayController {
 		res[constants.RemoteGatewayClassName] = constants.UnmanagedGatewayController
 	}
 
+	if features.EnableAgentgateway {
+		res[constants.AgentgatewayClassName] = constants.ManagedAgentgatewayController
+	}
+
 	if features.EnableAmbientWaypoints {
 		res[constants.WaypointGatewayClassName] = constants.ManagedGatewayMeshController
 	}
@@ -207,6 +210,17 @@ func getClassInfos() map[gateway.GatewayController]classInfo {
 			controllerLabel: constants.ManagedGatewayMeshControllerLabel,
 		}
 	}
+	if features.EnableAgentgateway {
+		m[constants.ManagedAgentgatewayController] = classInfo{
+			controller:          constants.ManagedAgentgatewayController,
+			description:         "Istio with Agentgateway.",
+			templates:           "agentgateway",
+			defaultServiceType:  corev1.ServiceTypeLoadBalancer,
+			addressType:         gateway.HostnameAddressType,
+			controllerLabel:     constants.ManagedGatewayControllerLabel,
+			supportsListenerSet: true,
+		}
+	}
 
 	if features.EnableAmbientMultiNetwork {
 		m[constants.ManagedGatewayEastWestController] = classInfo{
@@ -562,7 +576,7 @@ func (d *DeploymentController) configureIstioGateway(log *istiolog.Scope, gw gat
 		return nil
 	}
 	for _, t := range rendered {
-		if err := d.apply(gi.controller, t); err != nil {
+		if err := d.apply(gi.controller, t, input); err != nil {
 			return fmt.Errorf("apply failed: %v", err)
 		}
 	}
@@ -614,7 +628,7 @@ func translateInfraMeta[K ~string, V ~string](meta map[K]V) map[string]string {
 	return infra
 }
 
-func extractInfrastructureMetadata(gwInfra *gatewayv1.GatewayInfrastructure, isLabel bool, gw gateway.Gateway) map[string]string {
+func extractInfrastructureMetadata(gwInfra *gateway.GatewayInfrastructure, isLabel bool, gw gateway.Gateway) map[string]string {
 	if gwInfra != nil && isLabel && gwInfra.Labels != nil {
 		return translateInfraMeta(gwInfra.Labels)
 	}
@@ -688,10 +702,11 @@ type derivedInput struct {
 	TemplateInput
 
 	// Inserted from injection config
-	ProxyImage  string
-	ProxyConfig *meshapi.ProxyConfig
-	MeshConfig  *meshapi.MeshConfig
-	Values      map[string]any
+	ProxyImage        string
+	AgentgatewayImage string
+	ProxyConfig       *meshapi.ProxyConfig
+	MeshConfig        *meshapi.MeshConfig
+	Values            map[string]any
 }
 
 func (d *DeploymentController) render(templateName string, mi TemplateInput) ([]string, error) {
@@ -732,6 +747,11 @@ func (d *DeploymentController) render(templateName string, mi TemplateInput) ([]
 			proxyConfig.GetImage(),
 			mi.Annotations,
 		),
+		AgentgatewayImage: inject.AgentgatewayImage(
+			cfg.Values.Struct(),
+			proxyConfig.GetImage(),
+			mi.Annotations,
+		),
 		ProxyConfig: proxyConfig,
 		MeshConfig:  cfg.MeshConfig,
 		Values:      cfg.Values.Map(),
@@ -867,7 +887,7 @@ func fetchParameters(gw *gateway.Gateway) (*types.NamespacedName, error) {
 }
 
 func (d *DeploymentController) setGatewayControllerVersion(gws gateway.Gateway) error {
-	patch := fmt.Sprintf(`{"apiVersion":"gateway.networking.k8s.io/v1beta1","kind":"Gateway","metadata":{"annotations":{"%s":"%d"}}}`,
+	patch := fmt.Sprintf(`{"apiVersion":"gateway.networking.k8s.io/v1","kind":"Gateway","metadata":{"annotations":{"%s":"%d"}}}`,
 		ControllerVersionAnnotation, ControllerVersion)
 
 	log.Debugf("applying %v", patch)
@@ -877,13 +897,38 @@ func (d *DeploymentController) setGatewayControllerVersion(gws gateway.Gateway)
 }
 
 // apply server-side applies a template to the cluster.
-func (d *DeploymentController) apply(controller string, yml string) error {
+func (d *DeploymentController) apply(controller string, yml string, input TemplateInput) error {
 	data := map[string]any{}
 	err := yaml.Unmarshal([]byte(yml), &data)
 	if err != nil {
 		return err
 	}
 	us := unstructured.Unstructured{Object: data}
+
+	// safeguard: validate object type matches default template
+	// only allow the 5 kinds defined in kube-gateway.yaml
+	kind := us.GetKind()
+	allowedKinds := sets.New("Deployment", "Service", "ServiceAccount", "HorizontalPodAutoscaler", "PodDisruptionBudget")
+	if !allowedKinds.Contains(kind) {
+		return fmt.Errorf("unexpected object kind %q, only %v are allowed", kind, allowedKinds.UnsortedList())
+	}
+
+	// safeguard: validate object namespace matches gateway namespace
+	objNamespace := us.GetNamespace()
+	if objNamespace != input.Namespace {
+		return fmt.Errorf("object namespace %q does not match gateway namespace %q", objNamespace, input.Namespace)
+	}
+
+	// safeguard: validate object name matches expected names from template
+	// expected names: DeploymentName for most resources, ServiceAccount for SA
+	objName := us.GetName()
+	expectedNames := sets.New(input.DeploymentName, input.ServiceAccount)
+	// also accept gateway name as prefix for backward compat
+	validName := expectedNames.Contains(objName) || strings.HasPrefix(objName, input.Name)
+	if !validName {
+		return fmt.Errorf("object name %q does not match expected pattern (expected %v or prefix of %q)", objName, expectedNames.UnsortedList(), input.Name)
+	}
+
 	// set managed-by label
 	clabel := strings.ReplaceAll(controller, "/", "-")
 	err = unstructured.SetNestedField(us.Object, clabel, "metadata", "labels", label.GatewayManaged.Name)
