@@ -22,11 +22,11 @@ import (
 
 	"go.uber.org/atomic"
 	corev1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	inferencev1 "sigs.k8s.io/gateway-api-inference-extension/api/v1"
 	gatewayv1 "sigs.k8s.io/gateway-api/apis/v1"
 	gatewayalpha "sigs.k8s.io/gateway-api/apis/v1alpha2"
-	gateway "sigs.k8s.io/gateway-api/apis/v1beta1"
 
 	istio "istio.io/api/networking/v1alpha3"
 	networkingclient "istio.io/client-go/pkg/apis/networking/v1"
@@ -35,6 +35,7 @@ import (
 	"istio.io/istio/pkg/config/constants"
 	"istio.io/istio/pkg/config/gateway/kube"
 	"istio.io/istio/pkg/config/schema/gvk"
+	"istio.io/istio/pkg/config/schema/kind"
 	"istio.io/istio/pkg/kube/controllers"
 	"istio.io/istio/pkg/kube/krt"
 	"istio.io/istio/pkg/ptr"
@@ -45,29 +46,36 @@ import (
 type AncestorBackend struct {
 	Gateway types.NamespacedName
 	Backend TypedNamespacedName
+	Source  TypedNamespacedName
 }
 
 func (a AncestorBackend) Equals(other AncestorBackend) bool {
-	return a.Gateway == other.Gateway && a.Backend == other.Backend
+	return a.Gateway == other.Gateway && a.Backend == other.Backend && a.Source == other.Source
 }
 
 func (a AncestorBackend) ResourceName() string {
-	return a.Gateway.String() + "/" + a.Backend.String()
+	return a.Source.String() + "/" + a.Gateway.String() + "/" + a.Backend.String()
 }
 
 func HTTPRouteCollection(
-	httpRoutes krt.Collection[*gateway.HTTPRoute],
+	httpRoutes krt.Collection[*gatewayv1.HTTPRoute],
 	inputs RouteContextInputs,
 	opts krt.OptionsBuilder,
-) RouteResult[*gateway.HTTPRoute, gateway.HTTPRouteStatus] {
+) RouteResult[*gatewayv1.HTTPRoute, gatewayv1.HTTPRouteStatus] {
 	routeCount := gatewayRouteAttachmentCountCollection(inputs, httpRoutes, gvk.HTTPRoute, opts)
-	ancestorBackends := krt.NewManyCollection(httpRoutes, func(krtctx krt.HandlerContext, obj *gateway.HTTPRoute) []AncestorBackend {
-		return extractAncestorBackends(obj.Namespace, obj.Spec.ParentRefs, obj.Spec.Rules, func(r gateway.HTTPRouteRule) []gateway.HTTPBackendRef {
-			return r.BackendRefs
-		})
+	ancestorBackends := krt.NewManyCollection(httpRoutes, func(krtctx krt.HandlerContext, obj *gatewayv1.HTTPRoute) []AncestorBackend {
+		return extractAncestorBackends(
+			obj.ObjectMeta,
+			kind.FromString(obj.Kind),
+			obj.Spec.ParentRefs,
+			obj.Spec.Rules,
+			func(r gatewayv1.HTTPRouteRule) []gatewayv1.HTTPBackendRef {
+				return r.BackendRefs
+			},
+		)
 	}, opts.WithName("HTTPAncestors")...)
-	status, baseVirtualServices := krt.NewStatusManyCollection(httpRoutes, func(krtctx krt.HandlerContext, obj *gateway.HTTPRoute) (
-		*gateway.HTTPRouteStatus,
+	status, baseVirtualServices := krt.NewStatusManyCollection(httpRoutes, func(krtctx krt.HandlerContext, obj *gatewayv1.HTTPRoute) (
+		*gatewayv1.HTTPRouteStatus,
 		[]RouteWithKey,
 	) {
 		ctx := inputs.WithCtx(krtctx)
@@ -77,7 +85,7 @@ func HTTPRouteCollection(
 		}{}
 		status := obj.Status.DeepCopy()
 		route := obj.Spec
-		parentStatus, parentRefs, meshResult, gwResult := computeRoute(ctx, obj, func(mesh bool, obj *gateway.HTTPRoute) iter.Seq2[*istio.HTTPRoute, *ConfigError] {
+		parentStatus, parentRefs, meshResult, gwResult := computeRoute(ctx, obj, func(mesh bool, obj *gatewayv1.HTTPRoute) iter.Seq2[*istio.HTTPRoute, *ConfigError] {
 			return func(yield func(*istio.HTTPRoute, *ConfigError) bool) {
 				for n, r := range route.Rules {
 					// split the rule to make sure each rule has up to one match
@@ -87,7 +95,7 @@ func HTTPRouteCollection(
 					}
 					for _, m := range matches {
 						if m != nil {
-							r.Matches = []gateway.HTTPRouteMatch{*m}
+							r.Matches = []gatewayv1.HTTPRouteMatch{*m}
 						}
 						istioRoute, ipCfg, configErr := convertHTTPRoute(ctx, r, obj, n, !mesh)
 						if istioRoute != nil && ipCfg != nil && ipCfg.enableExtProc {
@@ -128,7 +136,7 @@ func HTTPRouteCollection(
 					routeKey += "/" + strconv.Itoa(int(*parent.OriginalReference.Port))
 				}
 				ref := types.NamespacedName{
-					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gateway.Namespace(obj.Namespace))),
+					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gatewayv1.Namespace(obj.Namespace))),
 					Name:      string(parent.OriginalReference.Name),
 				}
 				if parent.InternalKind == gvk.ServiceEntry {
@@ -143,7 +151,7 @@ func HTTPRouteCollection(
 					vsHosts = []string{
 						string(parent.OriginalReference.Name) +
 							"." +
-							string(ptr.OrDefault(parent.OriginalReference.Namespace, gateway.Namespace(obj.Namespace))) +
+							string(ptr.OrDefault(parent.OriginalReference.Namespace, gatewayv1.Namespace(obj.Namespace))) +
 							".svc." + ctx.DomainSuffix,
 					}
 				}
@@ -204,7 +212,7 @@ func HTTPRouteCollection(
 	}, opts.WithName("HTTPRoute")...)
 
 	finalVirtualServices := mergeHTTPRoutes(baseVirtualServices, opts.WithName("HTTPRouteMerged")...)
-	return RouteResult[*gateway.HTTPRoute, gateway.HTTPRouteStatus]{
+	return RouteResult[*gatewayv1.HTTPRoute, gatewayv1.HTTPRouteStatus]{
 		VirtualServices:  finalVirtualServices,
 		RouteAttachments: routeCount,
 		Status:           status,
@@ -212,7 +220,21 @@ func HTTPRouteCollection(
 	}
 }
 
-func extractAncestorBackends[RT, BT any](ns string, prefs []gateway.ParentReference, rules []RT, extract func(RT) []BT) []AncestorBackend {
+func extractAncestorBackends[RT, BT any](
+	obj metav1.ObjectMeta,
+	kind kind.Kind,
+	prefs []gatewayv1.ParentReference,
+	rules []RT,
+	extract func(RT) []BT,
+) []AncestorBackend {
+	ns := obj.Namespace
+	source := TypedNamespacedName{
+		NamespacedName: types.NamespacedName{
+			Namespace: obj.Namespace,
+			Name:      obj.Name,
+		},
+		Kind: kind,
+	}
 	gateways := sets.Set[types.NamespacedName]{}
 	for _, r := range prefs {
 		ref := normalizeReference(r.Group, r.Kind, gvk.KubernetesGateway)
@@ -250,6 +272,7 @@ func extractAncestorBackends[RT, BT any](ns string, prefs []gateway.ParentRefere
 			res = append(res, AncestorBackend{
 				Gateway: gw,
 				Backend: be,
+				Source:  source,
 			})
 		}
 	}
@@ -268,9 +291,15 @@ func GRPCRouteCollection(
 ) RouteResult[*gatewayv1.GRPCRoute, gatewayv1.GRPCRouteStatus] {
 	routeCount := gatewayRouteAttachmentCountCollection(inputs, grpcRoutes, gvk.GRPCRoute, opts)
 	ancestorBackends := krt.NewManyCollection(grpcRoutes, func(krtctx krt.HandlerContext, obj *gatewayv1.GRPCRoute) []AncestorBackend {
-		return extractAncestorBackends(obj.Namespace, obj.Spec.ParentRefs, obj.Spec.Rules, func(r gatewayv1.GRPCRouteRule) []gatewayv1.GRPCBackendRef {
-			return r.BackendRefs
-		})
+		return extractAncestorBackends(
+			obj.ObjectMeta,
+			kind.FromString(obj.Kind),
+			obj.Spec.ParentRefs,
+			obj.Spec.Rules,
+			func(r gatewayv1.GRPCRouteRule) []gatewayv1.GRPCBackendRef {
+				return r.BackendRefs
+			},
+		)
 	}, opts.WithName("GRPCAncestors")...)
 	status, baseVirtualServices := krt.NewStatusManyCollection(grpcRoutes, func(krtctx krt.HandlerContext, obj *gatewayv1.GRPCRoute) (
 		*gatewayv1.GRPCRouteStatus,
@@ -325,7 +354,7 @@ func GRPCRouteCollection(
 					routeKey += "/" + strconv.Itoa(int(*parent.OriginalReference.Port))
 				}
 				ref := types.NamespacedName{
-					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gateway.Namespace(obj.Namespace))),
+					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gatewayv1.Namespace(obj.Namespace))),
 					Name:      string(parent.OriginalReference.Name),
 				}
 				if parent.InternalKind == gvk.ServiceEntry {
@@ -338,7 +367,7 @@ func GRPCRouteCollection(
 					}
 				} else {
 					vsHosts = []string{fmt.Sprintf("%s.%s.svc.%s",
-						parent.OriginalReference.Name, ptr.OrDefault(parent.OriginalReference.Namespace, gateway.Namespace(obj.Namespace)), ctx.DomainSuffix)}
+						parent.OriginalReference.Name, ptr.OrDefault(parent.OriginalReference.Namespace, gatewayv1.Namespace(obj.Namespace)), ctx.DomainSuffix)}
 				}
 			}
 			if len(routes) == 0 {
@@ -412,9 +441,15 @@ func TCPRouteCollection(
 ) RouteResult[*gatewayalpha.TCPRoute, gatewayalpha.TCPRouteStatus] {
 	routeCount := gatewayRouteAttachmentCountCollection(inputs, tcpRoutes, gvk.TCPRoute, opts)
 	ancestorBackends := krt.NewManyCollection(tcpRoutes, func(krtctx krt.HandlerContext, obj *gatewayalpha.TCPRoute) []AncestorBackend {
-		return extractAncestorBackends(obj.Namespace, obj.Spec.ParentRefs, obj.Spec.Rules, func(r gatewayalpha.TCPRouteRule) []gateway.BackendRef {
-			return r.BackendRefs
-		})
+		return extractAncestorBackends(
+			obj.ObjectMeta,
+			kind.FromString(obj.Kind),
+			obj.Spec.ParentRefs,
+			obj.Spec.Rules,
+			func(r gatewayalpha.TCPRouteRule) []gatewayv1.BackendRef {
+				return r.BackendRefs
+			},
+		)
 	}, opts.WithName("TCPAncestors")...)
 	status, virtualServices := krt.NewStatusManyCollection(tcpRoutes, func(krtctx krt.HandlerContext, obj *gatewayalpha.TCPRoute) (
 		*gatewayalpha.TCPRouteStatus,
@@ -446,7 +481,7 @@ func TCPRouteCollection(
 					routes = augmentTCPPortMatch(routes, *parent.OriginalReference.Port)
 				}
 				ref := types.NamespacedName{
-					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gateway.Namespace(obj.Namespace))),
+					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gatewayv1.Namespace(obj.Namespace))),
 					Name:      string(parent.OriginalReference.Name),
 				}
 				if parent.InternalKind == gvk.ServiceEntry {
@@ -503,9 +538,15 @@ func TLSRouteCollection(
 ) RouteResult[*gatewayalpha.TLSRoute, gatewayalpha.TLSRouteStatus] {
 	routeCount := gatewayRouteAttachmentCountCollection(inputs, tlsRoutes, gvk.TLSRoute, opts)
 	ancestorBackends := krt.NewManyCollection(tlsRoutes, func(krtctx krt.HandlerContext, obj *gatewayalpha.TLSRoute) []AncestorBackend {
-		return extractAncestorBackends(obj.Namespace, obj.Spec.ParentRefs, obj.Spec.Rules, func(r gatewayalpha.TLSRouteRule) []gateway.BackendRef {
-			return r.BackendRefs
-		})
+		return extractAncestorBackends(
+			obj.ObjectMeta,
+			kind.FromString(obj.Kind),
+			obj.Spec.ParentRefs,
+			obj.Spec.Rules,
+			func(r gatewayalpha.TLSRouteRule) []gatewayv1.BackendRef {
+				return r.BackendRefs
+			},
+		)
 	}, opts.WithName("TLSAncestors")...)
 	status, virtualServices := krt.NewStatusManyCollection(tlsRoutes, func(krtctx krt.HandlerContext, obj *gatewayalpha.TLSRoute) (
 		*gatewayalpha.TLSRouteStatus,
@@ -534,7 +575,7 @@ func TLSRouteCollection(
 			if parent.IsMesh() {
 				routes = meshResult.routes
 				ref := types.NamespacedName{
-					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gateway.Namespace(obj.Namespace))),
+					Namespace: string(ptr.OrDefault(parent.OriginalReference.Namespace, gatewayv1.Namespace(obj.Namespace))),
 					Name:      string(parent.OriginalReference.Name),
 				}
 				if parent.InternalKind == gvk.ServiceEntry {
@@ -591,7 +632,7 @@ func computeRoute[T controllers.Object, O comparable](ctx RouteContext, obj T, t
 	mesh bool,
 	obj T,
 ) iter.Seq2[O, *ConfigError],
-) ([]gateway.RouteParentStatus, []routeParentReference, conversionResult[O], conversionResult[O]) {
+) ([]gatewayv1.RouteParentStatus, []routeParentReference, conversionResult[O], conversionResult[O]) {
 	parentRefs := extractParentReferenceInfo(ctx, ctx.RouteParents, obj)
 
 	convertRules := func(mesh bool) conversionResult[O] {
@@ -771,12 +812,63 @@ func mergeHTTPRoutes(baseVirtualServices krt.Collection[RouteWithKey], opts ...k
 		sortRoutesByCreationTime(configs)
 		base := configs[0].DeepCopy()
 		baseVS := base.Spec.(*istio.VirtualService)
-		for _, config := range configs[1:] {
+		// Deep copy the InferencePool configs map to avoid race conditions
+		// The default DeepCopy() only does shallow copy of Extra field
+		if base.Extra != nil {
+			if ipConfigs, ok := base.Extra[constants.ConfigExtraPerRouteRuleInferencePoolConfigs].(map[string]kube.InferencePoolRouteRuleConfig); ok {
+				// Create a new map to avoid modifying the shared underlying map
+				newIPConfigs := make(map[string]kube.InferencePoolRouteRuleConfig, len(ipConfigs))
+				for k, v := range ipConfigs {
+					newIPConfigs[k] = v
+				}
+				base.Extra[constants.ConfigExtraPerRouteRuleInferencePoolConfigs] = newIPConfigs
+			}
+		}
+		for i, config := range configs[1:] {
 			thisVS := config.Spec.(*istio.VirtualService)
 			baseVS.Http = append(baseVS.Http, thisVS.Http...)
 			// append parents
 			base.Annotations[constants.InternalParentNames] = fmt.Sprintf("%s,%s",
 				base.Annotations[constants.InternalParentNames], config.Annotations[constants.InternalParentNames])
+			// Merge Extra field (especially for InferencePool configs)
+			if base.Extra == nil && config.Extra != nil {
+				base.Extra = make(map[string]any)
+			}
+			if config.Extra != nil {
+				for k, v := range config.Extra {
+					// For non-InferencePool configs, keep the first value for stability
+					if k != constants.ConfigExtraPerRouteRuleInferencePoolConfigs {
+						if _, exists := base.Extra[k]; !exists {
+							base.Extra[k] = v
+						}
+						continue
+					}
+					// For InferencePool configs, merge the maps
+					baseMap, baseOk := base.Extra[k].(map[string]kube.InferencePoolRouteRuleConfig)
+					configMap, configOk := v.(map[string]kube.InferencePoolRouteRuleConfig)
+					if baseOk && configOk {
+						log.Debugf("Merging InferencePool configs: adding %d route configs from VirtualService %d to base (namespace=%s)",
+							len(configMap), i+1, config.Namespace)
+						// Route names are composed of the HTTPRoute/VirtualService namespaced name so they can't possibly conflict
+						for routeName, routeConfig := range configMap {
+							baseMap[routeName] = routeConfig
+						}
+					} else if configOk {
+						if _, exists := base.Extra[k]; !exists {
+							log.Debugf("Creating new InferencePool config map from VirtualService %d (namespace=%s)", i+1, config.Namespace)
+							base.Extra[k] = v
+						}
+					} else if !configOk {
+						log.Debugf("Skipping InferencePool config from VirtualService %d due to unexpected type (namespace=%s)", i+1, config.Namespace)
+					}
+				}
+			}
+		}
+		// Log final merged InferencePool configs
+		if base.Extra != nil {
+			if ipConfigs, ok := base.Extra[constants.ConfigExtraPerRouteRuleInferencePoolConfigs].(map[string]kube.InferencePoolRouteRuleConfig); ok {
+				log.Debugf("Final merged VirtualService for key %s has %d InferencePool route configs", object.Key, len(ipConfigs))
+			}
 		}
 		sortHTTPRoutes(baseVS.Http)
 		base.Name = strings.ReplaceAll(object.Key, "/", "~")
