@@ -17,8 +17,6 @@ package memory
 import (
 	"fmt"
 
-	"k8s.io/apimachinery/pkg/types"
-
 	"istio.io/istio/pilot/pkg/model"
 	"istio.io/istio/pkg/config"
 	"istio.io/istio/pkg/config/schema/collection"
@@ -123,24 +121,6 @@ func (c *Controller) UpdateStatus(config config.Config) (newRevision string, err
 	return newRevision, err
 }
 
-func (c *Controller) Patch(orig config.Config, patchFn config.PatchFunc) (newRevision string, err error) {
-	cfg, typ := patchFn(orig.DeepCopy())
-	switch typ {
-	case types.MergePatchType:
-	case types.JSONPatchType:
-	default:
-		return "", fmt.Errorf("unsupported merge type: %s", typ)
-	}
-	if newRevision, err = c.configStore.Patch(cfg, patchFn); err == nil {
-		c.monitor.ScheduleProcessEvent(ConfigEvent{
-			old:    orig,
-			config: cfg,
-			event:  model.EventUpdate,
-		})
-	}
-	return newRevision, err
-}
-
 func (c *Controller) Delete(kind config.GroupVersionKind, key, namespace string, resourceVersion *string) error {
 	if config := c.Get(kind, key, namespace); config != nil {
 		if err := c.configStore.Delete(kind, key, namespace, resourceVersion); err != nil {
