@@ -31,6 +31,7 @@ import (
 	"google.golang.org/protobuf/types/known/structpb"
 	"k8s.io/apimachinery/pkg/types"
 
+	"istio.io/api/label"
 	meshconfig "istio.io/api/mesh/v1alpha1"
 	"istio.io/istio/pilot/pkg/credentials"
 	"istio.io/istio/pilot/pkg/features"
@@ -1087,3 +1088,32 @@ func OutboundListenerClass(t NodeType) istionetworking.ListenerClass {
 	}
 	return istionetworking.ListenerClassSidecarOutbound
 }
+
+func IsIngressGateway(proxy *Proxy) bool {
+	if proxy == nil || proxy.Type != Router {
+		return false
+	}
+
+	return proxy.Labels[label.GatewayManaged.Name] == constants.ManagedGatewayControllerLabel ||
+		proxy.Labels[constants.IstioLabel] == constants.IstioIngressLabelValue // This is a legacy label
+}
+
+func IsWaypointProxy(node *Proxy) bool {
+	if node == nil || node.Type != Waypoint {
+		return false
+	}
+	controller, isManagedGateway := node.Labels[label.GatewayManaged.Name]
+
+	return isManagedGateway && controller == constants.ManagedGatewayMeshControllerLabel
+}
+
+func ShouldCreateDoubleHBONEResources(p *Proxy) bool {
+	isHBONESendEnabled := bool(!p.Metadata.DisableHBONESend) && features.EnableHBONESend
+
+	// Note that we only consider EnableHBONESend for ingress gateway, as traditionally
+	// that flag has been ignored for waypoints when generating endpoint/cluster discovery
+	// information.
+	return features.EnableAmbientMultiNetwork &&
+		(IsIngressGateway(p) && features.EnableAmbientIngressMultiNetwork && isHBONESendEnabled) ||
+		(IsWaypointProxy(p) && features.EnableAmbientWaypointMultiNetwork)
+}
