@@ -267,6 +267,17 @@ func (configgen *ConfigGeneratorImpl) buildClusters(proxy *model.Proxy, req *mod
 			clusters = append(clusters, configgen.buildOutboundSniDnatClusters(proxy, req, patcher)...)
 		}
 		clusters = append(clusters, patcher.insertedClusters()...)
+		// Ingress gateway needs the clusters necessary for Double HBONE communications
+		// that happen cross cluster. A request arrives at the ingress and the LB
+		// configuration may straightaway redirect the request to a backend in a
+		// remote cluster/network.
+		if model.ShouldCreateDoubleHBONEResources(proxy) {
+			clusters = append(
+				clusters,
+				cb.buildInnerConnectOriginateCluster(proxy, req.Push),
+				cb.buildOuterConnectOriginateCluster(proxy, req.Push),
+			)
+		}
 	}
 
 	// OutboundTunnel cluster is needed for sidecar and gateway.
