@@ -71,7 +71,8 @@ const (
 	// We use this header to indicate to the next proxy (e.g., E/W gateway), that the request
 	// came from a waypoint and the next proxy then can infer that L7 policies have been
 	// applied already.
-	downstreamSourceHeader = "x-istio-source"
+	downstreamSourceHeader        = "x-istio-source"
+	downstreamOriginNetworkHeader = "x-forwarded-network"
 )
 
 func (lb *ListenerBuilder) serviceForHostname(name host.Name) *model.Service {
@@ -106,10 +107,10 @@ func (lb *ListenerBuilder) buildWaypointInbound() []*listener.Listener {
 		listeners = append(listeners, buildWaypointConnectOriginateListener(lb.push, lb.node))
 	}
 
-	if features.EnableAmbientMultiNetwork && features.EnableAmbientWaypointMultiNetwork && isWaypointProxy(lb.node) {
+	if model.ShouldCreateDoubleHBONEResources(lb.node) {
 		listeners = append(listeners,
-			buildWaypointInnerConnectOriginateListener(lb.push, lb.node),
-			buildWaypointOuterConnectOriginateListener(lb.push, lb.node))
+			buildInnerConnectOriginateListener(lb.push, lb.node),
+			buildOuterConnectOriginateListener(lb.push, lb.node))
 	}
 	return listeners
 }
@@ -174,7 +175,7 @@ func (lb *ListenerBuilder) buildHCMConnectTerminateChain(routes []*route.Route)
 		filters = authzBuilder.BuildTCPRulesAsHTTPFilter()
 	}
 	filters = append(filters,
-		xdsfilters.WaypointDownstreamMetadataFilter,
+		xdsfilters.GenerateWaypointDownstreamMetadataFilter(),
 		xdsfilters.ConnectAuthorityFilter)
 
 	// This filter checks whether the request went through a waypoint already and thefore whether L7 policies have
@@ -433,6 +434,11 @@ func (lb *ListenerBuilder) buildWaypointInternal(wls []model.WorkloadInfo, svcs
 						portProtocols[port.Port] = protocol.HTTP
 					}
 				} else {
+					if port.Protocol.IsTLS() && svc.Hostname.IsWildCarded() && svc.Resolution == model.DynamicDNS &&
+						!features.EnableWildcardHostServiceEntriesForTLS {
+						log.Debugf("skipping waypoint inbound chain for TLS protocol for service %s since the feature is disabled", svc.Hostname)
+						continue
+					}
 					// Note that this could include double hbone
 					chains = append(chains, tcpChain)
 					portMapper.Map[portString] = match.ToChain(tcpChain.Name)
@@ -571,6 +577,42 @@ func (lb *ListenerBuilder) buildWaypointInternal(wls []model.WorkloadInfo, svcs
 		return nil
 	}()
 
+	// by default match IPs first
+	primaryMatcher := &matcher.Matcher{
+		MatcherType: &matcher.Matcher_MatcherTree_{
+			MatcherTree: &matcher.Matcher_MatcherTree{
+				Input: match.DestinationIP,
+				TreeType: &matcher.Matcher_MatcherTree_CustomMatch{
+					CustomMatch: &xds.TypedExtensionConfig{
+						Name:        "ip",
+						TypedConfig: protoconv.MessageToAny(ipMatcher),
+					},
+				},
+			},
+		},
+	}
+
+	// when multi-network is enabled, prefer the hostnames for matching
+	// VIPs may overlap across networks, especially auto-generated ServiceEntry VIPs
+	// and if we hit an incorrect VIP match we may attempt to route to the wrong service
+	if len(svcHostnameMap.Map) > 0 && features.EnableAmbientMultiNetwork {
+		primaryMatcher = &matcher.Matcher{
+			OnNoMatch: &matcher.Matcher_OnMatch{
+				OnMatch: &matcher.Matcher_OnMatch_Matcher{
+					Matcher: primaryMatcher,
+				},
+			},
+			MatcherType: &matcher.Matcher_MatcherTree_{
+				MatcherTree: &matcher.Matcher_MatcherTree{
+					Input: match.AuthorityFilterStateInput,
+					TreeType: &matcher.Matcher_MatcherTree_ExactMatchMap{
+						ExactMatchMap: svcHostnameMap,
+					},
+				},
+			},
+		}
+	}
+
 	l := &listener.Listener{
 		Name:              MainInternalName,
 		ListenerSpecifier: &listener.Listener_InternalListener{InternalListener: &listener.Listener_InternalListenerConfig{}},
@@ -578,39 +620,11 @@ func (lb *ListenerBuilder) buildWaypointInternal(wls []model.WorkloadInfo, svcs
 			xdsfilters.OriginalDestination,
 			httpInspector,
 		},
-		TrafficDirection: core.TrafficDirection_INBOUND,
-		FilterChains:     chains,
-		FilterChainMatcher: &matcher.Matcher{
-			MatcherType: &matcher.Matcher_MatcherTree_{
-				MatcherTree: &matcher.Matcher_MatcherTree{
-					Input: match.DestinationIP,
-					TreeType: &matcher.Matcher_MatcherTree_CustomMatch{
-						CustomMatch: &xds.TypedExtensionConfig{
-							Name:        "ip",
-							TypedConfig: protoconv.MessageToAny(ipMatcher),
-						},
-					},
-				},
-			},
-		},
+		TrafficDirection:   core.TrafficDirection_INBOUND,
+		FilterChains:       chains,
+		FilterChainMatcher: primaryMatcher,
 	}
 
-	if len(svcHostnameMap.Map) > 0 && features.EnableAmbientMultiNetwork {
-		l.FilterChainMatcher.OnNoMatch = &matcher.Matcher_OnMatch{
-			OnMatch: &matcher.Matcher_OnMatch_Matcher{
-				Matcher: &matcher.Matcher{
-					MatcherType: &matcher.Matcher_MatcherTree_{
-						MatcherTree: &matcher.Matcher_MatcherTree{
-							Input: match.AuthorityFilterStateInput,
-							TreeType: &matcher.Matcher_MatcherTree_ExactMatchMap{
-								ExactMatchMap: svcHostnameMap,
-							},
-						},
-					},
-				},
-			},
-		}
-	}
 	if tlsInspector != nil {
 		l.ListenerFilters = append(l.ListenerFilters, tlsInspector)
 	}
@@ -674,6 +688,32 @@ func buildConnectForwarder(push *model.PushContext, proxy *model.Proxy, class is
 		})
 	}
 
+	var filters []*listener.Filter
+
+	if features.EnableAmbientBaggage && tunnel {
+		// Make headers from the response available in the filter state for
+		// consumption by WaypointListenerBaggagePeerMetadata (peer_metadata filter).
+		tcpProxy.TunnelingConfig.PropagateResponseHeaders = true
+		tcpProxy.TunnelingConfig.HeadersToAdd = []*core.HeaderValueOption{
+			{
+				AppendAction: core.HeaderValueOption_OVERWRITE_IF_EXISTS_OR_ADD,
+				Header: &core.HeaderValue{
+					Key:   "baggage",
+					Value: "%FILTER_STATE(io.istio.baggage:PLAIN)%",
+				},
+			},
+		}
+		filters = []*listener.Filter{xdsfilters.WaypointListenerBaggagePeerMetadata}
+	}
+
+	tcp := &listener.Filter{
+		Name: wellknown.TCPProxy,
+		ConfigType: &listener.Filter_TypedConfig{
+			TypedConfig: protoconv.MessageToAny(tcpProxy),
+		},
+	}
+	filters = append(filters, tcp)
+
 	l := &listener.Listener{
 		Name:              clusterName,
 		UseOriginalDst:    wrappers.Bool(false),
@@ -682,144 +722,13 @@ func buildConnectForwarder(push *model.PushContext, proxy *model.Proxy, class is
 			xdsfilters.OriginalDestination,
 		},
 		FilterChains: []*listener.FilterChain{{
-			Filters: []*listener.Filter{{
-				Name: wellknown.TCPProxy,
-				ConfigType: &listener.Filter_TypedConfig{
-					TypedConfig: protoconv.MessageToAny(tcpProxy),
-				},
-			}},
+			Filters: filters,
 		}},
 	}
 	accessLogBuilder.setListenerAccessLog(push, proxy, l, class)
 	return l
 }
 
-// buildWaypointInnerConnectOriginateListener creates configuration for the waypoint inner_connect_originate listener.
-// This listener together with the corresponding cluster is responsible for the creating the inner HBONE tunnel of the
-// double-HBONE connection needed to communicate across networks in ambient mode.
-//
-// In terms of Envoy configuration this listener has two important bits:
-//
-// 1. It uses TCP Proxy network filter to establish a CONNECT tunnel
-// 2. It captures and propagates metadata about the actual destination and service name required for double-HBONE.
-func buildWaypointInnerConnectOriginateListener(push *model.PushContext, proxy *model.Proxy) *listener.Listener {
-	setFilterState := &sfsnetwork.Config{
-		OnNewConnection: []*sfsvalue.FilterStateValue{
-			{
-				Key: &sfsvalue.FilterStateValue_ObjectKey{
-					ObjectKey: "envoy.filters.listener.original_dst.local_ip",
-				},
-				Value: &sfsvalue.FilterStateValue_FormatString{
-					FormatString: &core.SubstitutionFormatString{
-						Format: &core.SubstitutionFormatString_TextFormatSource{
-							TextFormatSource: &core.DataSource{
-								Specifier: &core.DataSource_InlineString{
-									InlineString: "%DYNAMIC_METADATA(envoy.filters.listener.original_dst:local)%",
-								},
-							},
-						},
-					},
-				},
-				SharedWithUpstream: sfsvalue.FilterStateValue_TRANSITIVE,
-			},
-			{
-				Key: &sfsvalue.FilterStateValue_ObjectKey{
-					ObjectKey: "istio.double_hbone.hbone_target_address",
-				},
-				Value: &sfsvalue.FilterStateValue_FormatString{
-					FormatString: &core.SubstitutionFormatString{
-						Format: &core.SubstitutionFormatString_TextFormatSource{
-							TextFormatSource: &core.DataSource{
-								Specifier: &core.DataSource_InlineString{
-									InlineString: "%DYNAMIC_METADATA(istio:double_hbone:hbone_target_address)%",
-								},
-							},
-						},
-					},
-				},
-				FactoryKey:         "envoy.string",
-				SharedWithUpstream: sfsvalue.FilterStateValue_TRANSITIVE,
-			},
-		},
-	}
-	tcpProxy := &tcp.TcpProxy{
-		StatPrefix:       DoubleHBONEInnerConnectOriginate,
-		ClusterSpecifier: &tcp.TcpProxy_Cluster{Cluster: DoubleHBONEInnerConnectOriginate},
-		TunnelingConfig: &tcp.TcpProxy_TunnelingConfig{
-			Hostname: "%FILTER_STATE(istio.double_hbone.hbone_target_address:PLAIN)%",
-		},
-	}
-	accessLogBuilder.setHboneOriginationAccessLog(push, proxy, tcpProxy, istionetworking.ListenerClassSidecarInbound)
-
-	l := &listener.Listener{
-		Name:              DoubleHBONEInnerConnectOriginate,
-		UseOriginalDst:    wrappers.Bool(false),
-		ListenerSpecifier: &listener.Listener_InternalListener{InternalListener: &listener.Listener_InternalListenerConfig{}},
-		FilterChains: []*listener.FilterChain{{
-			Filters: []*listener.Filter{
-				{
-					Name: "propagate_endpoint_metadata",
-					ConfigType: &listener.Filter_TypedConfig{
-						TypedConfig: protoconv.MessageToAny(setFilterState),
-					},
-				},
-				{
-					Name: wellknown.TCPProxy,
-					ConfigType: &listener.Filter_TypedConfig{
-						TypedConfig: protoconv.MessageToAny(tcpProxy),
-					},
-				},
-			},
-		}},
-	}
-	accessLogBuilder.setListenerAccessLog(push, proxy, l, istionetworking.ListenerClassSidecarInbound)
-	return l
-}
-
-// buildWaypointOuterConnectOriginateListener creates configuration for the waypoint outer_connect_originate listener.
-// This listener together with the corresponding cluster is responsible for the creating the outer HBONE tunnel of the
-// double-HBONE connection needed to communicate across networks in ambient mode.
-//
-// Just like the inner_connect_originate listener above it uses TCP Proxy network filter to wrap traffic in HTTP CONNECT
-// tunnel, but unlike the inner_connect_originate it does not need to capture any metadata - it uses what
-// inner_connect_originate already captured and put in the filter state.
-func buildWaypointOuterConnectOriginateListener(push *model.PushContext, proxy *model.Proxy) *listener.Listener {
-	tcpProxy := &tcp.TcpProxy{
-		StatPrefix:       DoubleHBONEOuterConnectOriginate,
-		ClusterSpecifier: &tcp.TcpProxy_Cluster{Cluster: DoubleHBONEOuterConnectOriginate},
-		TunnelingConfig: &tcp.TcpProxy_TunnelingConfig{
-			Hostname: "%FILTER_STATE(istio.double_hbone.hbone_target_address:PLAIN)%",
-			HeadersToAdd: []*core.HeaderValueOption{{
-				AppendAction: core.HeaderValueOption_OVERWRITE_IF_EXISTS_OR_ADD,
-				Header: &core.HeaderValue{
-					Key:   downstreamSourceHeader,
-					Value: "waypoint",
-				},
-			}},
-		},
-	}
-	accessLogBuilder.setHboneOriginationAccessLog(push, proxy, tcpProxy, istionetworking.ListenerClassSidecarInbound)
-
-	l := &listener.Listener{
-		Name:              DoubleHBONEOuterConnectOriginate,
-		UseOriginalDst:    wrappers.Bool(false),
-		ListenerSpecifier: &listener.Listener_InternalListener{InternalListener: &listener.Listener_InternalListenerConfig{}},
-		ListenerFilters: []*listener.ListenerFilter{
-			xdsfilters.OriginalDestination,
-		},
-		FilterChains: []*listener.FilterChain{{
-			Filters: []*listener.Filter{{
-				Name: wellknown.TCPProxy,
-				ConfigType: &listener.Filter_TypedConfig{
-					TypedConfig: protoconv.MessageToAny(tcpProxy),
-				},
-			}},
-		}},
-	}
-	accessLogBuilder.setListenerAccessLog(push, proxy, l, istionetworking.ListenerClassSidecarInbound)
-	return l
-}
-
 // buildWaypointHTTPFilters augments the common chain of Waypoint-bound HTTP filters.
 // Authn/authz filters are prepended. Telemetry filters are appended.
 func (lb *ListenerBuilder) buildWaypointHTTPFilters(svc *model.Service) (pre []*hcm.HttpFilter, post []*hcm.HttpFilter) {
@@ -848,7 +757,7 @@ func (lb *ListenerBuilder) buildWaypointHTTPFilters(svc *model.Service) (pre []*
 	// TODO: these feel like the wrong place to insert, but this retains backwards compatibility with the original implementation
 	post = extension.PopAppendHTTP(post, wasm, extensions.PluginPhase_STATS)
 	post = extension.PopAppendHTTP(post, wasm, extensions.PluginPhase_UNSPECIFIED_PHASE)
-	post = append(post, xdsfilters.WaypointUpstreamMetadataFilter)
+	post = append(post, xdsfilters.GenerateWaypointUpstreamMetadataFilter())
 	post = append(post, lb.push.Telemetry.HTTPFilters(lb.node, cls, svc)...)
 	return pre, post
 }
@@ -916,6 +825,7 @@ func (lb *ListenerBuilder) buildWaypointNetworkFilters(svc *model.Service, fcc i
 		ClusterSpecifier: &tcp.TcpProxy_Cluster{Cluster: fcc.clusterName},
 	}
 	var destinationRule *networking.DestinationRule
+	var sniDFPFilter *listener.Filter
 	if svc != nil {
 		svcHostname = svc.Hostname
 		virtualServices := getVirtualServiceForWaypoint(lb.node.ConfigNamespace, svc, lb.node.SidecarScope.EgressListeners[0].VirtualServices())
@@ -953,6 +863,11 @@ func (lb *ListenerBuilder) buildWaypointNetworkFilters(svc *model.Service, fcc i
 			}
 			tcpProxy.ClusterSpecifier = clusterSpecifier
 		}
+
+		// Conditionally build SNI DFP for wildcard hosts with Dynamic DNS resolution
+		if svcHostname.IsWildCarded() && svc.Resolution == model.DynamicDNS {
+			sniDFPFilter = buildSNIDFPFilter(fcc.port.Port, svc)
+		}
 	}
 
 	// To align with sidecars, subsets are ignored here...?
@@ -968,6 +883,10 @@ func (lb *ListenerBuilder) buildWaypointNetworkFilters(svc *model.Service, fcc i
 
 	tcpFilter := setAccessLogAndBuildTCPFilter(lb.push, lb.node, tcpProxy, istionetworking.ListenerClassSidecarInbound, fcc.policyService)
 	networkFilterstack := buildNetworkFiltersStack(fcc.port.Protocol, tcpFilter, statPrefix, fcc.clusterName)
+	if sniDFPFilter != nil {
+		// Add SNI DFP before the TCP proxy so it can use SNI to resolve DNS
+		networkFilterstack = append([]*listener.Filter{sniDFPFilter}, networkFilterstack...)
+	}
 	return lb.buildCompleteNetworkFilters(istionetworking.ListenerClassSidecarInbound, fcc.port.Port, networkFilterstack, true, fcc.policyService)
 }
 
