@@ -18,6 +18,7 @@ import (
 	"fmt"
 	"net/url"
 	"sort"
+	"time"
 
 	core "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
 	tracingcfg "github.com/envoyproxy/go-control-plane/envoy/config/trace/v3"
@@ -27,6 +28,7 @@ import (
 	tracing "github.com/envoyproxy/go-control-plane/envoy/type/tracing/v3"
 	xdstype "github.com/envoyproxy/go-control-plane/envoy/type/v3"
 	"google.golang.org/protobuf/types/known/anypb"
+	"google.golang.org/protobuf/types/known/durationpb"
 	"google.golang.org/protobuf/types/known/wrapperspb"
 
 	meshconfig "istio.io/api/mesh/v1alpha1"
@@ -177,7 +179,8 @@ func configureFromProviderConfig(pushCtx *model.PushContext, proxy *model.Proxy,
 				return nil, fmt.Errorf("could not find cluster for tracing provider %q: %v", provider, err)
 			}
 			traceContextOption := convertTraceContextOption(provider.Zipkin.GetTraceContextOption())
-			return zipkinConfig(hostname, cluster, provider.Zipkin.GetPath(), !provider.Zipkin.GetEnable_64BitTraceId(), traceContextOption, proxy)
+			return zipkinConfig(hostname, cluster, provider.Zipkin.GetPath(), !provider.Zipkin.GetEnable_64BitTraceId(),
+				traceContextOption, provider.Zipkin.GetTimeout(), provider.Zipkin.GetHeaders(), proxy)
 		}
 	case *meshconfig.MeshConfig_ExtensionProvider_Datadog:
 		maxTagLength = provider.Datadog.GetMaxTagLength()
@@ -247,20 +250,60 @@ func convertTraceContextOption(
 
 func zipkinConfig(
 	hostname, cluster, endpoint string, enable128BitTraceID bool, traceContextOption tracingcfg.ZipkinConfig_TraceContextOption,
-	proxy *model.Proxy,
+	timeout *durationpb.Duration, headers []*meshconfig.MeshConfig_ExtensionProvider_HttpHeader, proxy *model.Proxy,
 ) (*anypb.Any, error) {
 	if endpoint == "" {
 		endpoint = "/api/v2/spans" // envoy deprecated v1 support
 	}
 	zc := &tracingcfg.ZipkinConfig{
-		CollectorCluster:         cluster,
-		CollectorEndpoint:        endpoint,
 		CollectorEndpointVersion: tracingcfg.ZipkinConfig_HTTP_JSON, // use v2 JSON for now
-		CollectorHostname:        hostname,                          // http host header
 		TraceId_128Bit:           enable128BitTraceID,               // istio default enable 128 bit trace id
 		SharedSpanContext:        wrapperspb.Bool(false),
 	}
 
+	// Determine if we should use HttpService (modern approach) or legacy fields
+	// For newer proxies (Istio 1.29+), always use HttpService as it's the modern approach
+	// that supports timeout and custom headers. For older proxies, use legacy fields for
+	// backward compatibility.
+	proxyVersionGreaterOrEqual129 := proxy.IstioVersion != nil && proxy.VersionGreaterOrEqual(&model.IstioVersion{Major: 1, Minor: 29})
+
+	if proxyVersionGreaterOrEqual129 {
+		// Modern configuration using HttpService
+		// This is required for timeout and custom headers support
+
+		// Set default timeout if not provided, as Envoy requires a timeout to be set.
+		if timeout == nil {
+			timeout = durationpb.New(5 * time.Second)
+		}
+
+		httpService := &core.HttpService{
+			HttpUri: &core.HttpUri{
+				Uri: fmt.Sprintf("http://%s%s", hostname, endpoint),
+				HttpUpstreamType: &core.HttpUri_Cluster{
+					Cluster: cluster,
+				},
+				Timeout: timeout,
+			},
+		}
+		// Add custom headers if provided
+		if len(headers) > 0 {
+			httpService.RequestHeadersToAdd = buildHTTPHeaders(headers)
+		}
+		zc.CollectorService = httpService
+	} else {
+		// Legacy configuration - maintains backward compatibility for older proxies
+		zc.CollectorCluster = cluster
+		zc.CollectorEndpoint = endpoint
+		zc.CollectorHostname = hostname // http host header
+
+		useHTTPService := timeout != nil || len(headers) > 0
+		// Log when timeout/headers configuration is ignored due to older proxy version
+		if useHTTPService {
+			log.Warnf("Proxy %s (version %v) does not support Zipkin timeout/headers configuration: requires Istio 1.29+. "+
+				"Configuration will be ignored.", proxy.ID, proxy.IstioVersion)
+		}
+	}
+
 	// Only set TraceContextOption for proxies that support it to avoid NACK from older proxies
 	// TraceContextOption support requires a recent Envoy version - using conservative version gating
 	if proxy.IstioVersion != nil && proxy.VersionGreaterOrEqual(&model.IstioVersion{Major: 1, Minor: 28}) {
@@ -506,11 +549,59 @@ var optionalPolicyTags = []*tracing.CustomTag{
 	dryRunPolicyTraceTag("istio.authorization.dry_run.deny_policy.result", authz_model.RBACShadowRulesDenyStatPrefix+authz_model.RBACShadowEngineResult),
 }
 
-func buildServiceTags(metadata *model.NodeMetadata, labels map[string]string) []*tracing.CustomTag {
+// buildWaypointSourceTags creates custom trace tags for waypoints that capture source (client) peer info.
+// Uses Envoy FILTER_STATE FIELD accessor to extract all available fields from downstream_peer_obj.
+// The peer_metadata filter stores WorkloadMetadataObject under downstream_peer_obj key,
+// while CelState is stored under downstream_peer key for CEL expression compatibility.
+// All fields are extracted individually to support dynamic addition of new fields in WorkloadMetadataObject.
+func buildWaypointSourceTags() []*tracing.CustomTag {
+	buildDownstreamFilterStateTag := func(tagName, fieldName string) *tracing.CustomTag {
+		return &tracing.CustomTag{
+			Tag: tagName,
+			Type: &tracing.CustomTag_Value{
+				Value: fmt.Sprintf("%%FILTER_STATE(downstream_peer_obj:FIELD:%s)%%", fieldName),
+			},
+		}
+	}
+	buildUpstreamFilterStateTag := func(tagName, fieldName string) *tracing.CustomTag {
+		return &tracing.CustomTag{
+			Tag: tagName,
+			Type: &tracing.CustomTag_Value{
+				Value: fmt.Sprintf("%%FILTER_STATE(upstream_peer_obj:FIELD:%s)%%", fieldName),
+			},
+		}
+	}
+
+	// Extract all available fields from WorkloadMetadataObject
+	// See: proxy/extensions/common/metadata_object.h for field definitions
+	return []*tracing.CustomTag{
+		buildUpstreamFilterStateTag("istio.destination_workload", "workload"),
+		buildUpstreamFilterStateTag("istio.destination_namespace", "namespace"),
+		buildUpstreamFilterStateTag("istio.destination_cluster_id", "cluster"),
+		buildUpstreamFilterStateTag("istio.destination_canonical_service", "service"),
+		buildUpstreamFilterStateTag("istio.destination_canonical_revision", "revision"),
+		buildUpstreamFilterStateTag("istio.destination_app", "app"),
+		buildUpstreamFilterStateTag("istio.destination_app_version", "version"),
+		buildUpstreamFilterStateTag("istio.destination_workload_type", "type"),
+		buildUpstreamFilterStateTag("istio.destination_instance_name", "name"),
+
+		buildDownstreamFilterStateTag("istio.source_workload", "workload"),
+		buildDownstreamFilterStateTag("istio.source_namespace", "namespace"),
+		buildDownstreamFilterStateTag("istio.source_cluster_id", "cluster"),
+		buildDownstreamFilterStateTag("istio.source_canonical_service", "service"),
+		buildDownstreamFilterStateTag("istio.source_canonical_revision", "revision"),
+		buildDownstreamFilterStateTag("istio.source_app", "app"),
+		buildDownstreamFilterStateTag("istio.source_app_version", "version"),
+		buildDownstreamFilterStateTag("istio.source_workload_type", "type"),
+		buildDownstreamFilterStateTag("istio.source_instance_name", "name"),
+	}
+}
+
+func buildServiceTags(node *model.Proxy) []*tracing.CustomTag {
 	var revision, service string
-	if labels != nil {
-		revision = labels["service.istio.io/canonical-revision"]
-		service = labels["service.istio.io/canonical-name"]
+	if node.Labels != nil {
+		revision = node.Labels["service.istio.io/canonical-revision"]
+		service = node.Labels["service.istio.io/canonical-name"]
 	}
 	if revision == "" {
 		revision = "latest"
@@ -519,19 +610,19 @@ func buildServiceTags(metadata *model.NodeMetadata, labels map[string]string) []
 	if service == "" {
 		service = "unknown"
 	}
-	meshID := metadata.MeshID
+	meshID := node.Metadata.MeshID
 	if meshID == "" {
 		meshID = "unknown"
 	}
-	namespace := metadata.Namespace
+	namespace := node.Metadata.Namespace
 	if namespace == "" {
 		namespace = "default"
 	}
-	clusterID := string(metadata.ClusterID)
+	clusterID := string(node.Metadata.ClusterID)
 	if clusterID == "" {
 		clusterID = "unknown"
 	}
-	return []*tracing.CustomTag{
+	tags := []*tracing.CustomTag{
 		{
 			Tag: "istio.canonical_revision",
 			Type: &tracing.CustomTag_Literal_{
@@ -573,6 +664,8 @@ func buildServiceTags(metadata *model.NodeMetadata, labels map[string]string) []
 			},
 		},
 	}
+
+	return tags
 }
 
 func configureSampling(hcmTracing *hcm.HttpConnectionManager_Tracing, providerPercentage float64) {
@@ -613,16 +706,21 @@ func configureCustomTags(spec *model.TracingSpec, hcmTracing *hcm.HttpConnection
 			enableIstioTags = proxyCfg.GetTracing().GetEnableIstioTags().GetValue()
 		}
 		if enableIstioTags {
-			tags = append(buildServiceTags(node.Metadata, node.Labels), optionalPolicyTags...)
+			tags = append(buildServiceTags(node), optionalPolicyTags...)
 		}
 	} else if spec.EnableIstioTags {
-		tags = append(buildServiceTags(node.Metadata, node.Labels), optionalPolicyTags...)
+		tags = append(buildServiceTags(node), optionalPolicyTags...)
+	}
+
+	// For waypoint proxies, add source peer tags to capture client workload info.
+	if node.Type == model.Waypoint {
+		tags = append(tags, buildWaypointSourceTags()...)
 	}
 
 	if len(providerTags) == 0 {
 		tags = append(tags, buildCustomTagsFromProxyConfig(proxyCfg.GetTracing().GetCustomTags())...)
 	} else {
-		tags = append(tags, buildCustomTagsFromProvider(providerTags)...)
+		tags = append(tags, buildCustomTagsFromProvider(node, providerTags)...)
 	}
 
 	// looping over customTags, a map, results in the returned value
@@ -635,48 +733,57 @@ func configureCustomTags(spec *model.TracingSpec, hcmTracing *hcm.HttpConnection
 	hcmTracing.CustomTags = tags
 }
 
-func buildCustomTagsFromProvider(providerTags map[string]*telemetrypb.Tracing_CustomTag) []*tracing.CustomTag {
+func buildCustomTagsFromProvider(node *model.Proxy, providerTags map[string]*telemetrypb.Tracing_CustomTag) []*tracing.CustomTag {
 	var tags []*tracing.CustomTag
+
+	supportFormatterTag := node.VersionGreaterOrEqual(&model.IstioVersion{Major: 1, Minor: 29, Patch: 0})
+	hasFormatterTag := false
+
 	for tagName, tagInfo := range providerTags {
 		if tagInfo == nil {
 			log.Warnf("while building custom tags from provider, encountered nil custom tag: %s, skipping", tagName)
 			continue
 		}
+		t := &tracing.CustomTag{
+			Tag: tagName,
+		}
 		switch tag := tagInfo.Type.(type) {
 		case *telemetrypb.Tracing_CustomTag_Environment:
-			env := &tracing.CustomTag{
-				Tag: tagName,
-				Type: &tracing.CustomTag_Environment_{
-					Environment: &tracing.CustomTag_Environment{
-						Name:         tag.Environment.Name,
-						DefaultValue: tag.Environment.DefaultValue,
-					},
+			t.Type = &tracing.CustomTag_Environment_{
+				Environment: &tracing.CustomTag_Environment{
+					Name:         tag.Environment.Name,
+					DefaultValue: tag.Environment.DefaultValue,
 				},
 			}
-			tags = append(tags, env)
 		case *telemetrypb.Tracing_CustomTag_Header:
-			header := &tracing.CustomTag{
-				Tag: tagName,
-				Type: &tracing.CustomTag_RequestHeader{
-					RequestHeader: &tracing.CustomTag_Header{
-						Name:         tag.Header.Name,
-						DefaultValue: tag.Header.DefaultValue,
-					},
+			t.Type = &tracing.CustomTag_RequestHeader{
+				RequestHeader: &tracing.CustomTag_Header{
+					Name:         tag.Header.Name,
+					DefaultValue: tag.Header.DefaultValue,
 				},
 			}
-			tags = append(tags, header)
 		case *telemetrypb.Tracing_CustomTag_Literal:
-			env := &tracing.CustomTag{
-				Tag: tagName,
-				Type: &tracing.CustomTag_Literal_{
-					Literal: &tracing.CustomTag_Literal{
-						Value: tag.Literal.Value,
-					},
+			t.Type = &tracing.CustomTag_Literal_{
+				Literal: &tracing.CustomTag_Literal{
+					Value: tag.Literal.Value,
 				},
 			}
-			tags = append(tags, env)
+		case *telemetrypb.Tracing_CustomTag_Formatter:
+			hasFormatterTag = true
+			if !supportFormatterTag {
+				continue
+			}
+			t.Type = &tracing.CustomTag_Value{
+				Value: tag.Formatter.Value,
+			}
 		}
+		tags = append(tags, t)
 	}
+
+	if !supportFormatterTag && hasFormatterTag {
+		log.Debug("Formatter custom tag only support for Istio 1.29+")
+	}
+
 	return tags
 }
 
