@@ -27,6 +27,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 
 	"istio.io/api/annotation"
+	"istio.io/api/label"
 	networkingv1alpha3 "istio.io/api/networking/v1alpha3"
 	networkingclient "istio.io/client-go/pkg/apis/networking/v1"
 	securityclient "istio.io/client-go/pkg/apis/security/v1"
@@ -39,7 +40,9 @@ import (
 	"istio.io/istio/pilot/pkg/util/protoconv"
 	"istio.io/istio/pkg/cluster"
 	"istio.io/istio/pkg/config/constants"
+	cfgkube "istio.io/istio/pkg/config/kube"
 	"istio.io/istio/pkg/config/labels"
+	"istio.io/istio/pkg/config/protocol"
 	"istio.io/istio/pkg/config/schema/gvk"
 	"istio.io/istio/pkg/config/schema/kind"
 	kubeutil "istio.io/istio/pkg/kube"
@@ -58,7 +61,7 @@ import (
 // WorkloadsCollection builds out the core Workload object type used in ambient mode.
 // A Workload represents a single addressable unit of compute -- typically a Pod or a VM.
 // Workloads can come from a variety of sources; these are joined together to build one complete `Collection[WorkloadInfo]`.
-func (a *index) WorkloadsCollection(
+func (a Builder) WorkloadsCollection(
 	pods krt.Collection[*v1.Pod],
 	nodes krt.Collection[Node],
 	meshConfig krt.Singleton[MeshConfig],
@@ -116,7 +119,8 @@ func (a *index) WorkloadsCollection(
 
 	NetworkGatewayWorkloads := krt.NewManyFromNothing[model.WorkloadInfo](func(ctx krt.HandlerContext) []model.WorkloadInfo {
 		meshCfg := krt.FetchOne(ctx, meshConfig.AsCollection())
-		return slices.Map(a.LookupAllNetworkGateway(ctx), convertGateway(meshCfg))
+		all := LookupAllNetworkGateway(ctx, a.NetworkGateways)
+		return slices.Map(all, convertGateway(meshCfg))
 	}, opts.WithName("NetworkGatewayWorkloads")...)
 
 	Workloads := krt.JoinCollection(
@@ -150,7 +154,7 @@ func MergedGlobalWorkloadsCollection(
 	localWorkloadServices krt.Collection[model.ServiceInfo],
 	globalWorkloadServices krt.Collection[krt.Collection[krt.ObjectWithCluster[model.ServiceInfo]]],
 	globalWorkloadServicesByCluster krt.Index[cluster.ID, krt.Collection[krt.ObjectWithCluster[model.ServiceInfo]]],
-	globalNetworks networkCollections,
+	globalNetworks NetworkCollections,
 	localClusterID cluster.ID,
 	flags FeatureFlags,
 	domainSuffix string,
@@ -704,7 +708,7 @@ func workloadEntryWorkloadBuilder(
 		w.WorkloadType = workloadapi.WorkloadType_POD // XXX(shashankram): HACK to impersonate pod
 		w.CanonicalName, w.CanonicalRevision = kubelabels.CanonicalService(wle.Labels, w.WorkloadName)
 
-		setTunnelProtocol(wle.Labels, wle.Annotations, w)
+		setTunnelProtocol(wle.Labels, wle.Annotations, w, flags)
 		localNetwork := localNetworkGetter(ctx)
 		if network != localNetwork.String() {
 			// This is a remote workload that we'll never send directly; don't precompute
@@ -725,7 +729,7 @@ func workloadEntryWorkloadBuilder(
 	}
 }
 
-func (a *index) workloadEntryWorkloadBuilder(
+func (a Builder) workloadEntryWorkloadBuilder(
 	meshConfig krt.Singleton[MeshConfig],
 	authorizationPolicies krt.Collection[model.WorkloadAuthorization],
 	peerAuths krt.Collection[*securityclient.PeerAuthentication],
@@ -750,8 +754,8 @@ func (a *index) workloadEntryWorkloadBuilder(
 			return a.ClusterID
 		},
 		localNetworkGetter,
-		a.networks.NetworkGateways,
-		a.networks.GatewaysByNetwork,
+		a.NetworkGateways,
+		a.GatewaysByNetwork,
 		a.Flags,
 	)
 }
@@ -874,7 +878,7 @@ func podWorkloadBuilder(
 		w.WorkloadType = workloadapi.WorkloadType_POD // backwards compatibility
 		w.CanonicalName, w.CanonicalRevision = kubelabels.CanonicalService(p.Labels, w.WorkloadName)
 
-		setTunnelProtocol(p.Labels, p.Annotations, w)
+		setTunnelProtocol(p.Labels, p.Annotations, w, flags)
 		localNetwork := localNetworkGetter(ctx)
 		if network != localNetwork.String() {
 			// This is a remote workload that we'll never send directly; don't precompute
@@ -894,7 +898,7 @@ func podWorkloadBuilder(
 	}
 }
 
-func (a *index) podWorkloadBuilder(
+func (a Builder) podWorkloadBuilder(
 	meshConfig krt.Singleton[MeshConfig],
 	authorizationPolicies krt.Collection[model.WorkloadAuthorization],
 	peerAuths krt.Collection[*securityclient.PeerAuthentication],
@@ -926,8 +930,8 @@ func (a *index) podWorkloadBuilder(
 			return a.ClusterID
 		},
 		localNetworkGetter,
-		a.networks.NetworkGateways,
-		a.networks.GatewaysByNetwork,
+		a.NetworkGateways,
+		a.GatewaysByNetwork,
 		a.Flags,
 	)
 }
@@ -1141,7 +1145,7 @@ func serviceEntryWorkloadBuilder(
 			w.WorkloadName, w.WorkloadType = se.Name, workloadapi.WorkloadType_POD // XXX(shashankram): HACK to impersonate pod
 			w.CanonicalName, w.CanonicalRevision = kubelabels.CanonicalService(se.Labels, w.WorkloadName)
 
-			setTunnelProtocol(se.Labels, se.Annotations, w)
+			setTunnelProtocol(se.Labels, se.Annotations, w, flags)
 			res = append(res, precomputeWorkload(model.WorkloadInfo{
 				Workload:     w,
 				Labels:       se.Labels,
@@ -1153,7 +1157,7 @@ func serviceEntryWorkloadBuilder(
 	}
 }
 
-func (a *index) serviceEntryWorkloadBuilder(
+func (a Builder) serviceEntryWorkloadBuilder(
 	meshConfig krt.Singleton[MeshConfig],
 	authorizationPolicies krt.Collection[model.WorkloadAuthorization],
 	peerAuths krt.Collection[*securityclient.PeerAuthentication],
@@ -1174,8 +1178,8 @@ func (a *index) serviceEntryWorkloadBuilder(
 		func(ctx krt.HandlerContext) network.ID {
 			return a.Network(ctx)
 		},
-		a.networks.NetworkGateways,
-		a.networks.GatewaysByNetwork,
+		a.NetworkGateways,
+		a.GatewaysByNetwork,
 		a.Flags,
 	)
 }
@@ -1319,7 +1323,7 @@ func endpointSlicesBuilder(
 	}
 }
 
-func (a *index) endpointSlicesBuilder(
+func (a Builder) endpointSlicesBuilder(
 	meshConfig krt.Singleton[MeshConfig],
 	workloadServices krt.Collection[model.ServiceInfo],
 ) krt.TransformationMulti[*discovery.EndpointSlice, model.WorkloadInfo] {
@@ -1336,7 +1340,10 @@ func (a *index) endpointSlicesBuilder(
 	)
 }
 
-func setTunnelProtocol(labels, annotations map[string]string, w *workloadapi.Workload) {
+func setTunnelProtocol(labels, annotations map[string]string, w *workloadapi.Workload, flags FeatureFlags) {
+	if flags.EnableMtlsTransportProtocol && labels[label.SecurityTlsMode.Name] == model.IstioMutualTLSModeLabel {
+		w.TunnelProtocol = workloadapi.TunnelProtocol_LEGACY_ISTIO_MTLS
+	}
 	if annotations[annotation.AmbientRedirection.Name] == constants.AmbientRedirectionEnabled {
 		// Configured for override
 		w.TunnelProtocol = workloadapi.TunnelProtocol_HBONE
@@ -1346,6 +1353,8 @@ func setTunnelProtocol(labels, annotations map[string]string, w *workloadapi.Wor
 		w.TunnelProtocol = workloadapi.TunnelProtocol_HBONE
 		w.NativeTunnel = true
 	}
+	// TODO: add TunnelProtocol_LEGACY_ISTIO_MTLS. This requires Ztunnel to support it for 1 version, though, since a new
+	// enum field would be rejected.
 }
 
 func pickTrustDomain(mesh *MeshConfig) string {
@@ -1664,3 +1673,19 @@ func precomputeWorkload(w model.WorkloadInfo) model.WorkloadInfo {
 	}
 	return w
 }
+
+func toAppProtocolFromKube(p v1.ServicePort) workloadapi.AppProtocol {
+	return toAppProtocolFromProtocol(cfgkube.ConvertProtocol(p.Port, p.Name, p.Protocol, p.AppProtocol))
+}
+
+func toAppProtocolFromProtocol(p protocol.Instance) workloadapi.AppProtocol {
+	switch p {
+	case protocol.HTTP:
+		return workloadapi.AppProtocol_HTTP11
+	case protocol.HTTP2:
+		return workloadapi.AppProtocol_HTTP2
+	case protocol.GRPC:
+		return workloadapi.AppProtocol_GRPC
+	}
+	return workloadapi.AppProtocol_UNKNOWN
+}
