@@ -38,6 +38,7 @@ import (
 	"istio.io/istio/pkg/cluster"
 	"istio.io/istio/pkg/config/schema/kind"
 	"istio.io/istio/pkg/env"
+	pm "istio.io/istio/pkg/model"
 	"istio.io/istio/pkg/util/sets"
 	"istio.io/istio/pkg/xds"
 )
@@ -346,9 +347,9 @@ func localityFromProxyLabels(proxy *model.Proxy) *core.Locality {
 	if !f1 && !f2 && !f3 {
 		// If no labels set, we didn't find the locality from the service registry. We do support a (mostly undocumented/internal)
 		// label to override the locality, so respect that here as well.
-		ls, f := proxy.Labels[model.LocalityLabel]
-		if f {
-			return util.ConvertLocality(ls)
+		localityLabel := pm.GetLocalityLabel(proxy.Labels)
+		if localityLabel != "" {
+			return util.ConvertLocality(localityLabel)
 		}
 		return nil
 	}
