@@ -946,12 +946,6 @@ func isEastWestGateway(node *model.Proxy) bool {
 	return isManagedGateway && controller == constants.ManagedGatewayEastWestControllerLabel
 }
 
-// Duplicated from networkin/core/waypoint to avoid circular dependency
-func isWaypointProxy(node *model.Proxy) bool {
-	if node == nil || node.Type != model.Waypoint {
-		return false
-	}
-	controller, isManagedGateway := node.Labels[label.GatewayManaged.Name]
-
-	return isManagedGateway && controller == constants.ManagedGatewayMeshControllerLabel
+func isSidecarProxy(node *model.Proxy) bool {
+	return node != nil && node.Type == model.SidecarProxy
 }
