@@ -199,6 +199,10 @@ func EnvoyMetricsServiceTCPKeepalive(value *networkingAPI.ConnectionPoolSettings
 	return newTCPKeepaliveOption("envoy_metrics_service_tcp_keepalive", value)
 }
 
+func EnvoyMetricsStatsCompression(value bool) Instance {
+	return newOption("envoy_metrics_stats_compression", value)
+}
+
 func EnvoyAccessLogServiceAddress(value string) Instance {
 	return newOptionOrSkipIfZero("envoy_accesslog_service_address", value).withConvert(addressConverter(value))
 }
@@ -232,6 +236,10 @@ func EnvoyStatusPort(value int) Instance {
 	return newOption("envoy_status_port", value)
 }
 
+func GlobalDownstreamMaxConnections(value int) Instance {
+	return newOption("global_downstream_max_connections", value)
+}
+
 func EnvoyStatusPortEnableProxyProtocol(value bool) Instance {
 	return newOption("envoy_status_port_enable_proxy_protocol", value)
 }
@@ -300,10 +308,6 @@ func EnvoyHistogramBuckets(value []HistogramBucket) Instance {
 	return newOption("histogram_buckets", value)
 }
 
-func EnvoyStatsCompression(value string) Instance {
-	return newOption("stats_compression", value)
-}
-
 func EnvoyStatsFlushInterval(interval time.Duration) Instance {
 	return newOption("stats_flush_interval", interval)
 }
