@@ -25,6 +25,7 @@ import (
 	"os"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/hashicorp/go-multierror"
@@ -98,6 +99,7 @@ import (
 	"istio.io/istio/pkg/log"
 	"istio.io/istio/pkg/sleep"
 	"istio.io/istio/pkg/test/util/yml"
+	"istio.io/istio/pkg/util/sets"
 	"istio.io/istio/pkg/version"
 )
 
@@ -160,6 +162,9 @@ type Client interface {
 
 	// ClusterID returns the cluster this client is connected to
 	ClusterID() cluster.ID
+
+	// IsWatchListSemanticsUnSupported is used by internal client-go libraries to tell if the client is a fake client (more or less)
+	IsWatchListSemanticsUnSupported() bool
 }
 
 // CLIClient is an extended client with additional helpers/functionality for Istioctl and testing.
@@ -207,6 +212,10 @@ type CLIClient interface {
 	// dynamically selected. If localAddress is empty, "localhost" is used.
 	NewPortForwarder(podName string, ns string, localAddress string, localPort int, podPort int) (PortForwarder, error)
 
+	// SetDefaultApplyNamespace sets a default namespace to deploy files to when using ApplyYAML on objects with no namespace
+	// set.
+	SetDefaultApplyNamespace(namespace string)
+
 	// ApplyYAMLFiles applies the resources in the given YAML files.
 	ApplyYAMLFiles(namespace string, yamlFiles ...string) error
 
@@ -438,6 +447,7 @@ func NewFakeClient(objects ...runtime.Object) CLIClient {
 	}
 
 	c.fastSync = true
+	c.isFake = true
 
 	c.version = lazy.NewWithRetry(c.kube.Discovery().ServerVersion)
 
@@ -454,6 +464,49 @@ func NewFakeClientWithVersion(minor string, objects ...runtime.Object) CLIClient
 	return c
 }
 
+// NewFakeClientWithNFailures creates a fake client that fails for get operations on the specified
+// resource kinds for the first failureCount attempts, then succeeds.
+// failingResources is a set of resource (e.g., "pods", "namespaces") that should fail.
+func NewFakeClientWithNFailures(failureCount int, failingResources sets.Set[schema.GroupVersionResource], objects ...runtime.Object) CLIClient {
+	c := NewFakeClient(objects...).(*client)
+
+	// Track retry attempts per resource
+	attempts := &sync.Map{}
+
+	// Create a reactor that fails initially then succeeds
+	retryReactor := func(action clienttesting.Action) (handled bool, ret runtime.Object, err error) {
+		// Only handle Get actions for failingResources
+		if action.GetVerb() != "get" {
+			return false, nil, nil
+		}
+		resource := action.GetResource()
+		if !failingResources.Contains(resource) {
+			return false, nil, nil
+		}
+
+		getAction := action.(clienttesting.GetAction)
+		key := fmt.Sprintf("%s/%s/%s", resource, action.GetNamespace(), getAction.GetName())
+
+		val, _ := attempts.LoadOrStore(key, atomic.NewInt32(0))
+		attemptCounter := val.(*atomic.Int32)
+
+		currentAttempt := attemptCounter.Inc()
+
+		if int(currentAttempt) <= failureCount {
+			return true, nil, kerrors.NewServiceUnavailable(fmt.Sprintf("transient error: attempt %d/%d", currentAttempt, failureCount))
+		}
+
+		return false, nil, nil
+	}
+
+	// Prepend the reactor for each failing kind
+	for resource := range failingResources {
+		c.kube.(*fake.Clientset).PrependReactor("get", resource.Resource, retryReactor)
+	}
+
+	return c
+}
+
 type fakeClient interface {
 	PrependReactor(verb, resource string, reaction clienttesting.ReactionFunc)
 	PrependWatchReactor(resource string, reaction clienttesting.WatchReactionFunc)
@@ -476,16 +529,19 @@ type client struct {
 	gatewayapi          gatewayapiclient.Interface
 	gatewayapiinference gatewayapiinferenceclient.Interface
 
+	isFake bool
+
 	started atomic.Bool
 	// If enabled, will wait for cache syncs with extremely short delay. This should be used only for tests
 	fastSync               bool
 	informerWatchesPending *atomic.Int32
 
 	// These may be set only when creating an extended client.
-	revision        string
-	restClient      *rest.RESTClient
-	discoveryClient discovery.CachedDiscoveryInterface
-	mapper          meta.ResettableRESTMapper
+	revision              string
+	restClient            *rest.RESTClient
+	discoveryClient       discovery.CachedDiscoveryInterface
+	mapper                meta.ResettableRESTMapper
+	defaultApplyNamespace string
 
 	version lazy.Lazy[*kubeVersion.Info]
 
@@ -765,6 +821,10 @@ func (c *client) ClusterID() cluster.ID {
 	return c.clusterID
 }
 
+func (c *client) IsWatchListSemanticsUnSupported() bool {
+	return c.isFake
+}
+
 // Wait for cache sync immediately, rather than with 100ms delay which slows tests
 // See https://github.com/kubernetes/kubernetes/issues/95262#issuecomment-703141573
 func fastWaitForCacheSync(stop <-chan struct{}, informerFactory informerfactory.InformerFactory) bool {
@@ -1166,6 +1226,10 @@ func (c *client) ServicesForSelector(ctx context.Context, namespace string, labe
 	})
 }
 
+func (c *client) SetDefaultApplyNamespace(namespace string) {
+	c.defaultApplyNamespace = namespace
+}
+
 func (c *client) ApplyYAMLFiles(namespace string, yamlFiles ...string) error {
 	g, _ := errgroup.WithContext(context.TODO())
 	for _, f := range removeEmptyFiles(yamlFiles) {
@@ -1388,6 +1452,9 @@ func (c *client) DynamicClientFor(g schema.GroupVersionKind, obj *unstructured.U
 		} else if namespace != "" && ns != namespace {
 			return nil, fmt.Errorf("object %v/%v provided namespace %q but apply called with %q", g, obj.GetName(), ns, namespace)
 		}
+		if ns == "" {
+			ns = c.defaultApplyNamespace
+		}
 		// namespaced resources should specify the namespace
 		dr = c.dynamic.Resource(gvr).Namespace(ns)
 	} else {
