@@ -19,8 +19,10 @@ import (
 	"net/netip"
 	"strconv"
 	"strings"
+	"unicode"
 
 	"github.com/hashicorp/go-multierror"
+	"k8s.io/apimachinery/pkg/api/resource"
 
 	"istio.io/api/annotation"
 	meshconfig "istio.io/api/mesh/v1alpha1"
@@ -47,6 +49,10 @@ var (
 		annotation.SidecarTrafficExcludeOutboundPorts.Name:        ValidateExcludeOutboundPorts,
 		annotation.PrometheusMergeMetrics.Name:                    validateBool,
 		annotation.ProxyConfig.Name:                               validateProxyConfig,
+		annotation.SidecarProxyCPU.Name:                           validateResourceQuantity,
+		annotation.SidecarProxyMemory.Name:                        validateResourceQuantity,
+		annotation.SidecarProxyCPULimit.Name:                      validateResourceQuantity,
+		annotation.SidecarProxyMemoryLimit.Name:                   validateResourceQuantity,
 	}
 )
 
@@ -145,6 +151,17 @@ func validateBool(value string) error {
 	return err
 }
 
+// validateResourceQuantity validates Kubernetes resource quantity - rejects control chars, validates format
+func validateResourceQuantity(value string) error {
+	for _, r := range value {
+		if unicode.IsControl(r) {
+			return fmt.Errorf("value contains control characters")
+		}
+	}
+	_, err := resource.ParseQuantity(value)
+	return err
+}
+
 func validateCIDRList(cidrs string) error {
 	if len(cidrs) > 0 {
 		for _, cidr := range strings.Split(cidrs, ",") {
