@@ -34,8 +34,9 @@ type mapCollection[T any, U any] struct {
 
 // nolint: unused // (not true, used in func declared to implement an interface)
 type mappedIndexer[T any, U any] struct {
-	indexer indexer[T]
-	mapFunc func(T) U
+	fromCollection string
+	indexer        indexer[T]
+	mapFunc        func(T) U
 }
 
 var _ Collection[any] = &mapCollection[any, any]{}
@@ -45,6 +46,9 @@ func (m *mappedIndexer[T, U]) Lookup(k string) []U {
 	keys := m.indexer.Lookup(k)
 	res := make([]U, 0, len(keys))
 	for _, obj := range keys {
+		if EnableAssertions {
+			assertKeyMatch(obj, m.mapFunc(obj), m.fromCollection)
+		}
 		res = append(res, m.mapFunc(obj))
 	}
 	return res
@@ -52,6 +56,9 @@ func (m *mappedIndexer[T, U]) Lookup(k string) []U {
 
 func (m *mapCollection[T, U]) GetKey(k string) *U {
 	if obj := m.collection.GetKey(k); obj != nil {
+		if EnableAssertions {
+			assertKeyMatch(*obj, m.mapFunc(*obj), m.collectionName)
+		}
 		return ptr.Of(m.mapFunc(*obj))
 	}
 	return nil
@@ -65,11 +72,7 @@ func (m *mapCollection[T, U]) List() []U {
 	}
 	if EnableAssertions {
 		for _, obj := range vals {
-			ok := GetKey(obj)
-			nk := GetKey(m.mapFunc(obj))
-			if nk != ok {
-				panic(fmt.Sprintf("Input and output key must be the same for MapCollection %q %q", ok, nk))
-			}
+			assertKeyMatch(obj, m.mapFunc(obj), m.collectionName)
 		}
 	}
 	return res
@@ -130,8 +133,9 @@ func (m *mapCollection[T, U]) index(name string, extract func(o U) []string) ind
 	}
 	idxs := m.collection.index(name, t)
 	return &mappedIndexer[T, U]{
-		indexer: idxs,
-		mapFunc: m.mapFunc,
+		fromCollection: m.collectionName,
+		indexer:        idxs,
+		mapFunc:        m.mapFunc,
 	}
 }
 
@@ -143,6 +147,8 @@ func (m *mapCollection[T, U]) WaitUntilSynced(stop <-chan struct{}) bool {
 	return m.collection.WaitUntilSynced(stop)
 }
 
+// MapCollection creates a new collection to map Collection[T] -> Collection[U]
+// Warning: It is not OK to use a mapFunc which changes how the Collection is keyed.
 func MapCollection[T, U any](
 	collection Collection[T],
 	mapFunc func(T) U,
@@ -167,3 +173,20 @@ func MapCollection[T, U any](
 	maybeRegisterCollectionForDebugging[U](m, o.debugger)
 	return m
 }
+
+// Used only to make assertions internally about Collection invariants
+// Will panic on violation. Ensure calls are wrapped by a check for `krt.EnableAssertions`
+func assertKeyMatch[T, U any](original T, mapped U, name string) {
+	originalKey := GetKey(original)
+	mappedKey := GetKey(mapped)
+	if originalKey != mappedKey {
+		msg := fmt.Sprintf("MapCollection invariant violation in %s: T and U must have matching keys: (%T)%s != (%T)%s",
+			name,
+			original,
+			originalKey,
+			mapped,
+			mappedKey,
+		)
+		panic(msg)
+	}
+}
