@@ -25,6 +25,7 @@ import (
 	matcher "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
 	"google.golang.org/protobuf/types/known/structpb"
 
+	apilabel "istio.io/api/label"
 	meshconfig "istio.io/api/mesh/v1alpha1"
 	"istio.io/istio/pilot/pkg/serviceregistry/util/label"
 	networkutil "istio.io/istio/pilot/pkg/util/network"
@@ -418,9 +419,28 @@ const (
 	k8sSeparator = "."
 )
 
-// GetLocalityLabel returns the locality from the supplied label. Because Kubernetes
+// GetLocalityLabel returns the locality label from the labels map.
+// It checks labels in the following priority order
+// 1. topology.istio.io/locality
+// 2. istio-locality (legacy label for backwards compatibility)
+// returns empty string if no locality label is found.
+func GetLocalityLabel(labels map[string]string) string {
+	// "topology.istio.io/locality" take first
+	if loc, ok := labels[apilabel.TopologyLocality.Name]; ok {
+		return loc
+	}
+
+	// check the old label(istio-locality)
+	if loc, ok := labels[LocalityLabel]; ok {
+		return loc
+	}
+
+	return ""
+}
+
+// SanitizeLocalityLabel returns the locality from the supplied label. Because Kubernetes
 // labels don't support `/`, we replace "." with "/" in the supplied label as a workaround.
-func GetLocalityLabel(label string) string {
+func SanitizeLocalityLabel(label string) string {
 	if len(label) > 0 {
 		// if there are /'s present we don't need to replace
 		if strings.Contains(label, "/") {
@@ -490,12 +510,15 @@ const (
 
 	// Ztunnel type is used for node proxies (ztunnel)
 	Ztunnel NodeType = "ztunnel"
+
+	// Agentgateway type is used for Agentgateway
+	Agentgateway NodeType = "agentgateway"
 )
 
 // IsApplicationNodeType verifies that the NodeType is one of the declared constants in the model
 func IsApplicationNodeType(nType NodeType) bool {
 	switch nType {
-	case SidecarProxy, Router, Waypoint, Ztunnel:
+	case SidecarProxy, Router, Waypoint, Ztunnel, Agentgateway:
 		return true
 	default:
 		return false
